/* Copyright(C) 2004 Brazil

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef _SENNA_H
#define _SENNA_H

#ifdef	__cplusplus
extern "C" {
#endif

#define SEN_INDEX_NORMALIZE     0x0001
#define SEN_INDEX_SPLIT_ALPHA   0x0002
#define SEN_INDEX_SPLIT_DIGIT   0x0004
#define SEN_INDEX_SPLIT_SYMBOL  0x0008
#define SEN_INDEX_NGRAM         0x0010

#ifndef SEN_STACK_SIZE
#define SEN_STACK_SIZE       0x10000000
#endif

typedef enum {
  sen_success = 0,
  sen_memory_exhausted,
  sen_invalid_format,
  sen_file_operation_error,
  sen_invalid_argument,
  sen_other_error
} sen_rc;

typedef enum {
  sen_enc_default = 0,
  sen_enc_none,
  sen_enc_euc_jp,
  sen_enc_utf8,
  sen_enc_sjis
} sen_encoding;

typedef enum {
  sen_rec_record = 0,
  sen_rec_section,
  sen_rec_position,
  sen_rec_userdef,
  sen_rec_none
} sen_rec_unit;

typedef enum {
  sen_sel_or = 0,
  sen_sel_and,
  sen_sel_but,
  sen_sel_adjust
} sen_sel_operator;

typedef enum {
  sen_sel_exact = 0,
  sen_sel_partial,
  sen_sel_unsplit,
  sen_sel_near,
  sen_sel_similar
} sen_sel_mode;

typedef enum {
  sen_sort_descending = 0,
  sen_sort_ascending
} sen_sort_mode;

typedef struct _sen_set sen_set;
typedef struct _sen_sym sen_sym;
typedef struct _sen_inv sen_inv;
typedef struct _sen_index sen_index;
typedef struct _sen_records sen_records;
typedef struct _sen_set_cursor sen_set_cursor;
typedef struct _sen_set_element * sen_set_eh;
typedef struct _sen_value sen_value;
typedef struct _sen_values sen_values;
typedef struct _sen_rec_element sen_rec_element;
typedef struct _sen_select_optarg sen_select_optarg;
typedef struct _sen_group_optarg sen_group_optarg;
typedef struct _sen_sort_optarg sen_sort_optarg;

typedef unsigned sen_id;
typedef sen_set_eh sen_recordh;

struct _sen_index {
  int foreign_keyp;
  sen_sym *keys;
  sen_sym *lexicon;
  sen_inv *inv;
};

struct _sen_records {
  sen_rec_unit record_unit;
  sen_rec_unit subrec_unit;
  unsigned int n_subrecs;
  unsigned int record_size;
  unsigned int subrec_size;
  sen_sym *keys;
  sen_set *records;
  sen_set_cursor *cursor;
  sen_recordh *curr_rec;
  int limit;
  sen_recordh *sorted;
};

struct _sen_rec_element {
  void *key;
  int score;
  unsigned int n_subrecs;
};

struct _sen_value {
  const char *str;
  unsigned int weight;
};

struct _sen_values {
  int n_values;
  sen_value *values;
};

struct _sen_select_optarg {
  sen_sel_mode mode;
  int similarity_threshold;
  int max_interval;
  int *weight_vector;
  int vector_size;
  int *(*mapper)(sen_records *, const void *, void *);
  void *mapper_arg;
};

struct _sen_group_optarg {
  sen_sort_mode mode;
  void *(*func)(sen_records *, const sen_recordh *, void *);
  void *func_arg;
  int key_size;
};

struct _sen_sort_optarg {
  sen_sort_mode mode;
  int (*compar)(sen_recordh *, sen_recordh *, void *);
  void *compar_arg;
};

/******** basic API ********/

sen_rc sen_init(void);

sen_index *sen_index_create(const char *path, int key_size,
			    int flags, int initial_n_segments, sen_encoding encoding);
sen_index *sen_index_open(const char *path);
sen_rc sen_index_info(sen_index *i, int *key_size,
		      int *flags, int *initial_n_segments, sen_encoding *encoding);
sen_rc sen_index_close(sen_index *i);
sen_rc sen_index_remove(const char *path);
sen_rc sen_index_upd(sen_index *i, const void *key,
                     const char *oldvalue, const char *newvalue);
sen_records *sen_index_sel(sen_index *i, const char *string);

const void * sen_records_next(sen_records *r);
sen_rc sen_records_rewind(sen_records *r);
int sen_records_curr_score(sen_records *r);
const void * sen_records_curr_key(sen_records *r);
int sen_records_nhits(sen_records *r);
int sen_records_find(sen_records *r, const void *key);
sen_rc sen_records_close(sen_records *r);

/******** advanced API ********/

sen_values *sen_values_open(void);
sen_rc sen_values_close(sen_values *v);
sen_rc sen_values_add(sen_values *v, const char *str, unsigned int weight);

sen_records *sen_records_open(sen_rec_unit record_unit,
			      sen_rec_unit subrec_unit, unsigned int n_subrecs);
sen_records *sen_records_union(sen_records *a, sen_records *b);
sen_records *sen_records_subtract(sen_records *a, sen_records *b);
sen_records *sen_records_intersect(sen_records *a, sen_records *b);
int sen_records_difference(sen_records *a, sen_records *b);
sen_rc sen_records_sort(sen_records *r, int limit, sen_sort_optarg *optarg);
sen_rc sen_records_group(sen_records *r, int limit, sen_group_optarg *optarg);
sen_rc sen_rec_info(sen_records *r, sen_recordh *rh, sen_rec_element *e);
int sen_rec_subrec_info(sen_records *r, sen_recordh *rh, sen_rec_element *e,
                                 int from, int limit);

sen_index *sen_index_create_with_keys(const char *path, sen_sym *keys,
			    int flags, int initial_n_segments, sen_encoding encoding);
sen_index *sen_index_open_with_keys(const char *path, sen_sym *keys);
sen_rc sen_index_update(sen_index *i, const void *key, unsigned int section,
                        sen_values *oldvalues, sen_values *newvalues);
sen_rc sen_index_select(sen_index *i, const char *string,
			sen_records *r, sen_sel_operator op, sen_select_optarg *optarg);

/******** low level API ********/

sen_set *sen_set_open(unsigned key_size, unsigned value_size, unsigned index_size);
sen_rc sen_set_close(sen_set *set);
sen_set_eh *sen_set_get(sen_set *set, const void *key, void **value);
sen_set_eh *sen_set_at(sen_set *set, const void *key, void **value);
sen_rc sen_set_del(sen_set *set, sen_set_eh *e);
sen_set_cursor *sen_set_cursor_open(sen_set *set);
sen_set_eh *sen_set_cursor_next(sen_set_cursor *cursor, void **key, void **value);
sen_rc sen_set_cursor_close(sen_set_cursor *cursor);

sen_set_eh *sen_set_sort(sen_set *set, int limit, sen_sort_optarg *optarg);
sen_rc sen_set_element_info(sen_set *set, const sen_set_eh *e, void **key, void **value);
sen_set *sen_set_union(sen_set *a, sen_set *b);
sen_set *sen_set_subtract(sen_set *a, sen_set *b);
sen_set *sen_set_intersect(sen_set *a, sen_set *b);
int sen_set_difference(sen_set *a, sen_set *b);

sen_sym * sen_sym_create(const char *path, unsigned key_size,
			 unsigned flags, sen_encoding encoding);
sen_sym * sen_sym_open(const char *path);
sen_rc sen_sym_close(sen_sym *sym);
sen_rc sen_sym_remove(const char *path);
sen_id sen_sym_get(sen_sym *sym, const unsigned char *key);
sen_id sen_sym_at(sen_sym *sym, const unsigned char *key);
sen_rc sen_sym_del(sen_sym *sym, const unsigned char *key);
unsigned int sen_sym_size(sen_sym *sym);
const unsigned char *sen_sym_key(sen_sym *sym, sen_id id);
sen_set * sen_sym_prefix_search(sen_sym *sym, const unsigned char *key);
sen_set * sen_sym_suffix_search(sen_sym *sym, const unsigned char *key);
sen_id sen_sym_common_prefix_search(sen_sym *sym, const unsigned char *key);

/* misc */

#define SEN_MALLOC malloc
#define SEN_REALLOC realloc
#define SEN_FREE free
#define SEN_MMAP(length,prot,flags,fd,offset) (mmap(0,length,prot,flags,fd,offset))
#define SEN_MUNMAP(start,length) (munmap(start,length))

void sen_log(char *fmt, ...);

sen_rc sen_index_check(sen_index *i);

#ifdef __cplusplus
}
#endif

#endif /* _SENNA_H */
